;;;
;;; Simon Kaplan Utilities
;;;
(require 'property)

;;; install the handlers
(push-property "EPOCH_SCROLL_UP_HALF" 'handle-epoch-scroll-up-half)
(push-property "EPOCH_SCROLL_DN_HALF" 'handle-epoch-scroll-dn-half)
(push-property "EPOCH_SCROLL_UP" 'handle-epoch-scroll-up)
(push-property "EPOCH_SCROLL_DN" 'handle-epoch-scroll-dn)
(push-property "EPOCH_SCROLL_UP_SLOW" 'handle-epoch-scroll-up-slow)
(push-property "EPOCH_SCROLL_DN_SLOW" 'handle-epoch-scroll-dn-slow)
(push-property "GWM_KILL_SCREEN"    'handle-gwm-kill-message)
(push-property "EPOCH_CHILD_MENU_REQUEST" 'handle-child-menu-request)
(push-property "EPOCH_PARENT_MENU_REQUEST" 'handle-parent-menu-request)
(push-property "EPOCH_BOTH_MENU_REQUEST" 'handle-both-menu-request)
(push-property "EPOCH_KEY_MENU_REQUEST" 'handle-keyword-menu-request)
(push-property "EPOCH_SIBLING_MENU_REQUEST" 'handle-sibling-menu-request)
(push-property "EPOCH_ALLDIRS_MENU_REQUEST" 'handle-alldirs-menu-request)
(push-property "MENU_CHOICE" 'handle-menu-choice)
(push-property "DIE" 'handle-die-request)
(push-property "MOVE_TO_SCREEN" 'handle-other-screen-move)
(push-property "EPOCH_NARROW" 'handle-epoch-narrow)

(defun handle-epoch-scroll-up (prop new scr)
  (select-screen scr)
  (if (> (count-lines (point-min) (point)) (screen-height))
      (scroll-down)
    (goto-char 0))
  (sit-for 0))

(defun handle-epoch-scroll-dn (prop new scr)
  (select-screen scr)
  (if (> (count-lines (point) (point-max)) (screen-height))
      (scroll-up)
    (goto-char (point-max)))
  (sit-for 0)
)

(defun handle-epoch-scroll-up-half (prop new scr)
  (select-screen scr)
  (if (> (count-lines (point-min) (point)) (/ (screen-height) 2))
      (unwind-protect
          (scroll-down (/ (screen-height) 2))
        (clear-mess))
    (goto-char 0))
  (sit-for 0))

(defun handle-epoch-scroll-dn-half (prop new scr)
  (select-screen scr)
  (if (> (count-lines (point) (point-max)) (/ (screen-height) 2))
      (unwind-protect
          (scroll-up (/ (screen-height) 2))
        (clear-mess))
    (goto-char (point-max)))
  (sit-for 0)
)



(defun handle-epoch-scroll-up-slow (prop new scr)
  (select-screen scr)
  (if (> (count-lines (point-min) (point)) 1)
      (scroll-down 1)
    (goto-char 0))
  (sit-for 0))

(defun handle-epoch-scroll-dn-slow (prop new scr)
  (select-screen scr)
  (if (> (count-lines (point) (point-max)) 1)
      (scroll-up 1)
    (goto-char 0))
  (sit-for 0))
 
(defun handle-gwm-kill-message (prop  new scr)
  (unwind-protect
      (delete-screen scr)
    (clear-mess))
  nil)

;;;
(defun list-current-window-buffers (window-list last-window current-window)
  (let ((this-window (next-window last-window)))
    (if (eq this-window current-window)
	window-list
      (list-current-windows
	(cons
	  (buffer-name (window-buffer this-window)) window-list
	)
	this-window
	current-window
      )
    )
  )
)

(defun list-windows ()
  "just an interface to list-current-windows and list-to-string."
  (list-to-string
   (make-string 0 ?x)
   (list-current-window-buffers 
    (list (buffer-name (window-buffer (selected-window))))
    (selected-window) (selected-window))))

;;;
(defun list-to-string (str lis)
  (if (null lis)
    str
    (mapconcat (function (lambda (x) x)) lis " ")
  )
)
